@@ -17,13 +17,123 @@
 #include "config.h"
 #endif
 
+/** 
+ * @page protocol MySQL Protocol
+ *
+ * The MySQL Protocol is spilt up into the four phases
+ *
+ * -# connect
+ * -# auth
+ * -# query
+ * -# and the close state
+ *
+ * @dot
+ * digraph states {
+ *   graph [rankdir=LR];
+ *   node [fontname=Helvetica, fontsize=10];
+ *
+ *   connect [ shape=record ];
+ *   close [ shape=record ];
+ *
+ *   subgraph { 
+ *     label = "client";
+ *     color = black;
+ *     rank = same;
+ *     node [ style=filled, fillcolor=lightblue ];
+ *     connect; 
+ *     auth; 
+ *     oldauth; 
+ *     query; 
+ *     local;
+ *   }
+ *
+ *   subgraph { 
+ *     label = "server";
+ *     rank = same; 
+ *     node [ style=filled, fillcolor=orange ];
+ *     handshake; 
+ *     authres; 
+ *     result; 
+ *     infile;
+ *   }
+ *   
+ *   subgraph { 
+ *     edge [ fontcolor=blue, color=blue, fontsize=10, fontname=Helvetica ];
+ *
+ *     connect->handshake [ label = "connecting server" ];
+ *     auth->authres [ label = "capabilities, password, default-db", URL="http://forge.mysql.com/wiki/MySQL_Internals_ClientServer_Protocol#Client_Authentication_Packet" ]; 
+ *     oldauth->authres [ label = "scrambled password" ] ; 
+ *     query->result [ label = "command (COM_*)", URL="http://forge.mysql.com/wiki/MySQL_Internals_ClientServer_Protocol#Command_Packet" ] ;
+ *     query->infile [ label = "LOAD DATA INFILE LOCAL" ];
+ *     local->result [ label = "file content"];
+ *   }
+ *
+ *   subgraph {
+ *     edge [ fontcolor=red, color=red, fontsize=10, fontname=Helvetica ];
+ *     handshake->close [ label = "ERR: host denied", URL="http://forge.mysql.com/wiki/MySQL_Internals_ClientServer_Protocol#Error_Packet" ];
+ *     handshake->auth [ label = "0x10: handshake", URL="http://forge.mysql.com/wiki/MySQL_Internals_ClientServer_Protocol#Handshake_Initialization_Packet" ];
+ *     authres->oldauth [ label = "EOF: old password reauth" ];
+ *     authres->query [ label = "OK: auth done", URL="http://forge.mysql.com/wiki/MySQL_Internals_ClientServer_Protocol#OK_Packet" ];
+ *     authres->close [ label = "ERR: auth failed", URL="http://forge.mysql.com/wiki/MySQL_Internals_ClientServer_Protocol#Error_Packet" ];
+ *     result->query [ label = "result for COM_*", URL="http://forge.mysql.com/wiki/MySQL_Internals_ClientServer_Protocol#Result_Set_Header_Packet" ] ;
+ *     result->close [ label = "COM_QUIT" ];
+ *     result->result [ label = "COM_BINLOG_DUMP" ];
+ *     infile->local [ label = "EOF: filename" ];
+ *   }
+ * }
+ * @enddot
+ * 
+ * Unfolded the sequence diagrams of the different use-cases:
+ *
+ * -# the client connects to the server and waits for data to return @msc
+ *   client, backend;
+ *   --- [ label = "connect to backend" ];
+ *   client->backend  [ label = "INIT" ];
+ * @endmsc
+ * -# the auth-phase handles the new SHA1-style passwords and the old scramble() passwords 
+ *   -# 4.1+ passwords @msc
+ *   client, backend;
+ *   --- [ label = "authenticate" ];
+ *   backend->client [ label = "HANDSHAKE" ];
+ *   client->backend [ label = "AUTH" ];
+ *   backend->client [ label = "AUTH_RESULT" ];
+ * @endmsc
+ *   -# pre-4.1 passwords @msc
+ *   client, backend;
+ *   --- [ label = "authenticate" ];
+ *   backend->client [ label = "HANDSHAKE" ];
+ *   client->backend [ label = "AUTH" ];
+ *   backend->client [ label = "OLD_PASSWORD_SCRAMBLE" ];
+ *   client->backend [ label = "OLD_PASSWORD_AUTH" ];
+ *   backend->client [ label = "AUTH_RESULT" ];
+ * @endmsc
+ * -# the query-phase repeats 
+ *   -# COM_QUERY and friends @msc
+ *   client, backend;
+ *   --- [ label = "query result phase" ];
+ *   client->backend [ label = "QUERY" ];
+ *   backend->client [ label = "QUERY_RESULT" ];
+ * @endmsc
+ *   -# COM_QUIT @msc
+ *   client, backend;
+ *   --- [ label = "query result phase" ];
+ *   client->backend [ label = "QUERY" ];
+ *   backend->client [ label = "connection close" ];
+ * @endmsc
+ *   -# COM_BINLOG_DUMP @msc
+ *   client, backend;
+ *   --- [ label = "query result phase" ];
+ *   client->backend [ label = "QUERY" ];
+ *   backend->client [ label = "QUERY_RESULT" ];
+ *   ... [ label = "more binlog entries" ];
+ *   backend->client [ label = "QUERY_RESULT"];
+ * @endmsc
+ */
+
 #include <sys/types.h>
 
 #ifdef HAVE_SYS_FILIO_H
-/**
- * required for FIONREAD on solaris
- */
-#include <sys/filio.h>
+#include <sys/filio.h> /* required for FIONREAD on solaris */
 #endif
 
 #ifndef _WIN32
@@ -68,9 +178,13 @@ extern volatile sig_atomic_t agent_shutdown;
 
 #ifdef _WIN32
 #define E_NET_CONNRESET WSAECONNRESET
+#define E_NET_CONNABORTED WSAECONNABORTED
 #define E_NET_WOULDBLOCK WSAEWOULDBLOCK
+#define E_NET_INPROGRESS WSAEINPROGRESS
 #else
 #define E_NET_CONNRESET ECONNRESET
+#define E_NET_CONNABORTED ECONNABORTED
+#define E_NET_INPROGRESS EINPROGRESS
 #if EWOULDBLOCK == EAGAIN
 /**
  * some system make EAGAIN == EWOULDBLOCK which would lead to a 
@@ -84,8 +198,19 @@ extern volatile sig_atomic_t agent_shutdown;
 #endif
 #endif
 
+/**
+ * a handy marco for constant strings 
+ */
 #define C(x) x, sizeof(x) - 1
 
+/**
+ * call the cleanup callback for the current connection
+ *
+ * @param srv    global context
+ * @param con    connection context
+ *
+ * @return       RET_SUCCESS on success
+ */
 retval_t plugin_call_cleanup(network_mysqld *srv, network_mysqld_con *con) {
 	NETWORK_MYSQLD_PLUGIN_FUNC(func) = NULL;
 
@@ -97,6 +222,12 @@ retval_t plugin_call_cleanup(network_mysqld *srv, network_mysqld_con *con) {
 }
 
 
+/**
+ * create a connection 
+ *
+ * @param srv    global context
+ * @return       a connection context
+ */
 network_mysqld_con *network_mysqld_con_init(network_mysqld *srv) {
 	network_mysqld_con *con;
 
@@ -109,6 +240,13 @@ network_mysqld_con *network_mysqld_con_init(network_mysqld *srv) {
 	return con;
 }
 
+/**
+ * free a connection 
+ *
+ * closes the client and server sockets 
+ *
+ * @param con    connection context
+ */
 void network_mysqld_con_free(network_mysqld_con *con) {
 	if (!con) return;
 
@@ -123,14 +261,21 @@ void network_mysqld_con_free(network_mysqld_con *con) {
 }
 
 
-
 /**
- * the free functions used by g_hash_table_new_full()
+ * type-safe free()-ing of the internal SQL tables
+ *
+ * @param s    a network_mysqld_tables 
+ * @see g_hash_table_new_full()
  */
 static void network_mysqld_tables_free_void(void *s) {
 	network_mysqld_table_free(s);
 }
 
+/**
+ * create a global context
+ *
+ * @see network_mysqld_tables_free_void()
+ */
 network_mysqld *network_mysqld_init() {
 	network_mysqld *m;
 
@@ -150,6 +295,7 @@ network_mysqld *network_mysqld_init() {
  *
  * kqueue has to be called after the fork() of daemonize
  *
+ * @param m      global context
  */
 void network_mysqld_init_libevent(network_mysqld *m) {
 	m->event_base  = event_init();
@@ -158,7 +304,9 @@ void network_mysqld_init_libevent(network_mysqld *m) {
 /**
  * free the global scope
  *
- * closes all open connections
+ * closes all open connections, cleans up all plugins
+ *
+ * @param m      global context
  */
 void network_mysqld_free(network_mysqld *m) {
 	guint i;
@@ -202,7 +350,17 @@ void network_mysqld_free(network_mysqld *m) {
 
 
 /**
- * connect to the proxy backend */
+ * translate a address-string into a network_address structure
+ *
+ * - if the address contains a colon we assume IPv4, 
+ *   - ":3306" -> (tcp) "0.0.0.0:3306"
+ * - if it starts with a / it is a unix-domain socket 
+ *   - "/tmp/socket" -> (unix) "/tmp/socket"
+ *
+ * @param addr     the address-struct
+ * @param address  the address string
+ * @return 0 on success, -1 otherwise
+ */
 int network_mysqld_con_set_address(network_address *addr, gchar *address) {
 	gchar *s;
 	guint port;
@@ -273,9 +431,37 @@ int network_mysqld_con_set_address(network_address *addr, gchar *address) {
 }
 
 /**
- * connect to the address defined in con->addr
+ * portable 'set non-blocking io'
  *
- * @see network_mysqld_set_address 
+ * @param fd      socket-fd
+ * @return        0
+ */
+int network_mysqld_con_set_non_blocking(int fd) {
+	int ioctlvar;
+
+#ifdef _WIN32
+	ioctlvar = 1;
+	if (0 != ioctlsocket(fd, FIONBIO, &ioctlvar)) {
+		errno = WSAGetLastError();
+		g_critical("%s.%d: ioctlsocket() failed: %s (%d)", 
+				__FILE__, __LINE__,
+				strerror(errno), errno);
+        }
+#else
+	fcntl(fd, F_SETFL, O_NONBLOCK | O_RDWR);
+#endif
+	return 0;
+}
+
+
+/**
+ * connect a socket
+ *
+ * the con->addr has to be set before 
+ * 
+ * @param con    a socket 
+ * @return       0 on connected, -1 on error, -2 for try again
+ * @see network_mysqld_set_address()
  */
 int network_mysqld_con_connect(network_socket * con) {
 	int val = 1;
@@ -287,31 +473,77 @@ int network_mysqld_con_connect(network_socket * con) {
 	 * even if it is not a IPv4 address as we use a union
 	 */
 	if (-1 == (con->fd = socket(con->addr.addr.ipv4.sin_family, SOCK_STREAM, 0))) {
+#ifdef _WIN32
+		errno = WSAGetLastError();
+#endif
 		g_critical("%s.%d: socket(%s) failed: %s", 
 				__FILE__, __LINE__,
 				con->addr.str, strerror(errno));
 		return -1;
 	}
 
-	setsockopt(con->fd, IPPROTO_TCP, TCP_NODELAY, &val, sizeof(val) );
+	/**
+	 * make the connect() call non-blocking
+	 *
+	 */
+	network_mysqld_con_set_non_blocking(con->fd);
 
 	if (-1 == connect(con->fd, (struct sockaddr *) &(con->addr.addr), con->addr.len)) {
-		g_critical("%s.%d: connect(%s) failed: %s", 
-				__FILE__, __LINE__,
-				con->addr.str,
-				strerror(errno));
-		return -1;
+		/**
+		 * in most TCP cases we connect() will return with 
+		 * EINPROGRESS ... 3-way handshake
+		 */
+
+#ifdef _WIN32
+		errno = WSAGetLastError();
+#endif
+		switch (errno) {
+		case E_NET_INPROGRESS:
+                case E_NET_WOULDBLOCK:
+			return -2;
+		default:
+			g_critical("%s.%d: connect(%s) failed: %s (%d)", 
+					__FILE__, __LINE__,
+					con->addr.str,
+					strerror(errno), errno);
+			return -1;
+		}
 	}
 
+	/**
+	 * set the same options as the mysql client 
+	 */
+#ifdef IP_TOS
+	val = 8;
+	setsockopt(con->fd, IPPROTO_IP,     IP_TOS, &val, sizeof(val));
+#endif
+	val = 1;
+	setsockopt(con->fd, IPPROTO_TCP,    TCP_NODELAY, &val, sizeof(val) );
+	val = 1;
+	setsockopt(con->fd, SOL_SOCKET, SO_KEEPALIVE, &val, sizeof(val) );
+
 	return 0;
 }
 
+
+/**
+ * connect a socket
+ *
+ * the con->addr has to be set before 
+ * 
+ * @param con    a socket 
+ * @return       0 on connected, -1 on error, -2 for try again
+ * @see network_mysqld_set_address()
+ */
 int network_mysqld_con_bind(network_socket * con) {
 	int val = 1;
 
 	g_assert(con->addr.len);
 
 	if (-1 == (con->fd = socket(con->addr.addr.ipv4.sin_family, SOCK_STREAM, 0))) {
+#ifdef _WIN32
+		errno = WSAGetLastError();
+#endif
 		g_critical("%s.%d: socket(%s) failed: %s", 
 				__FILE__, __LINE__,
 				con->addr.str, strerror(errno));
@@ -322,6 +554,9 @@ int network_mysqld_con_bind(network_socket * con) {
 	setsockopt(con->fd, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(val));
 
 	if (-1 == bind(con->fd, (struct sockaddr *) &(con->addr.addr), con->addr.len)) {
+#ifdef _WIN32
+		errno = WSAGetLastError();
+#endif
 		g_critical("%s.%d: bind(%s) failed: %s", 
 				__FILE__, __LINE__,
 				con->addr.str,
@@ -330,6 +565,9 @@ int network_mysqld_con_bind(network_socket * con) {
 	}
 
 	if (-1 == listen(con->fd, 8)) {
+#ifdef _WIN32
+		errno = WSAGetLastError();
+#endif
 		g_critical("%s.%d: listen() failed: %s",
 				__FILE__, __LINE__,
 				strerror(errno));
@@ -339,7 +577,7 @@ int network_mysqld_con_bind(network_socket * con) {
 	return 0;
 }
 
-
+#if 0 
 static void dump_str(const char *msg, const unsigned char *s, size_t len) {
 	GString *hex;
 	size_t i;
@@ -360,18 +598,26 @@ static void dump_str(const char *msg, const unsigned char *s, size_t len) {
 	g_message("(%s): %s", msg, hex->str);
 
 	g_string_free(hex, TRUE);
-
 }
+#endif
 
+/**
+ * create a OK packet and append it to the send-queue
+ *
+ * @param con             a client socket 
+ * @param affected_rows   affected rows 
+ * @param insert_id       insert_id 
+ * @param server_status   server_status (bitfield of SERVER_STATUS_*) 
+ * @param warnings        number of warnings to fetch with SHOW WARNINGS 
+ * @return 0
+ *
+ * @todo move to network_mysqld_proto
+ */
 int network_mysqld_con_send_ok_full(network_socket *con, guint64 affected_rows, guint64 insert_id, guint16 server_status, guint16 warnings ) {
 	GString *packet = g_string_new(NULL);
-	
-	network_mysqld_proto_append_int8(packet, 0); /* no fields */
-	network_mysqld_proto_append_lenenc_int(packet, affected_rows);
-	network_mysqld_proto_append_lenenc_int(packet, insert_id);
-	network_mysqld_proto_append_int16(packet, server_status); /* autocommit */
-	network_mysqld_proto_append_int16(packet, warnings); /* no warnings */
 
+	network_mysqld_proto_append_ok_packet(packet, affected_rows, insert_id, server_status, warnings);
+	
 	network_queue_append(con->send_queue, packet->str, packet->len, con->packet_id);
 
 	g_string_free(packet, TRUE);
@@ -379,8 +625,18 @@ int network_mysqld_con_send_ok_full(network_socket *con, guint64 affected_rows,
 	return 0;
 }
 
+/**
+ * send a simple OK packet
+ *
+ * - no affected rows
+ * - no insert-id
+ * - AUTOCOMMIT
+ * - no warnings
+ *
+ * @param con             a client socket 
+ */
 int network_mysqld_con_send_ok(network_socket *con) {
-	return network_mysqld_con_send_ok_full(con, 0, 0, 0x0002, 0);
+	return network_mysqld_con_send_ok_full(con, 0, 0, SERVER_STATUS_AUTOCOMMIT, 0);
 }
 
 /**
@@ -400,22 +656,8 @@ int network_mysqld_con_send_error_full(network_socket *con, const char *errmsg,
 	GString *packet;
 	
 	packet = g_string_sized_new(10 + errmsg_len);
-	
-	network_mysqld_proto_append_int8(packet, 0xff); /* ERR */
-	network_mysqld_proto_append_int16(packet, errorcode); /* errorcode */
-	g_string_append_c(packet, '#');
-	if (!sqlstate) {
-		g_string_append_len(packet, C("07000"));
-	} else {
-		g_string_append_len(packet, sqlstate, 5);
-	}
 
-	if (errmsg_len < 512) {
-		g_string_append_len(packet, errmsg, errmsg_len);
-	} else {
-		/* truncate the err-msg */
-		g_string_append_len(packet, errmsg, 512);
-	}
+	network_mysqld_proto_append_error_packet(packet, errmsg, errmsg_len, errorcode, sqlstate);
 
 	network_queue_append(con->send_queue, packet->str, packet->len, con->packet_id);
 
@@ -439,6 +681,10 @@ int network_mysqld_con_send_error(network_socket *con, const char *errmsg, gsize
 	return network_mysqld_con_send_error_full(con, errmsg, errmsg_len, ER_UNKNOWN_ERROR, NULL);
 }
 
+/**
+ * read a data from the socket
+ *
+ */
 retval_t network_mysqld_read_raw(network_mysqld *UNUSED_PARAM(srv), network_socket *con, GString *dest, size_t we_want) {
 	gssize len;
 
@@ -454,6 +700,7 @@ retval_t network_mysqld_read_raw(network_mysqld *UNUSED_PARAM(srv), network_sock
 		errno = WSAGetLastError();
 #endif
 		switch (errno) {
+		case E_NET_CONNABORTED:
 		case E_NET_CONNRESET: /** nothing to read, let's let ioctl() handle the close for us */
 		case E_NET_WOULDBLOCK: /** the buffers are empty, try again later */
 		case EAGAIN:     
@@ -511,7 +758,7 @@ retval_t network_mysqld_read(network_mysqld *srv, network_socket *con) {
 		con->packet_len = network_mysqld_proto_get_header((unsigned char *)(con->header->str));
 		con->packet_id  = (unsigned char)(con->header->str[3]); /* packet-id if the next packet */
 
-		packet = g_string_sized_new(con->packet_len + NET_HEADER_SIZE + 1); /** we need some space for the \0 */
+		packet = g_string_sized_new(con->packet_len + NET_HEADER_SIZE + 1); /* we need some space for the \0 */
 		g_string_append_len(packet, con->header->str, NET_HEADER_SIZE); /* copy the header */
 
 		network_queue_append_chunk(con->recv_queue, packet);
@@ -538,6 +785,10 @@ retval_t network_mysqld_read(network_mysqld *srv, network_socket *con) {
 	return RET_SUCCESS;
 }
 
+/**
+ * write data to the socket
+ *
+ */
 retval_t network_mysqld_write_len(network_mysqld *UNUSED_PARAM(srv), network_socket *con, int send_chunks) {
 	/* send the whole queue */
 	GList *chunk;
@@ -558,9 +809,16 @@ retval_t network_mysqld_write_len(network_mysqld *UNUSED_PARAM(srv), network_soc
 			case E_NET_WOULDBLOCK:
 			case EAGAIN:
 				return RET_WAIT_FOR_EVENT;
+			case EPIPE:
+			case E_NET_CONNRESET:
+			case E_NET_CONNABORTED:
+				/** remote side closed the connection */
+				return RET_ERROR;
 			default:
-				g_message("%s.%d: write() failed: %s", 
+				g_message("%s.%d: send(%s, %ld) failed: %s", 
 						__FILE__, __LINE__, 
+						con->addr.str, 
+						s->len - con->send_queue->offset, 
 						strerror(errno));
 				return RET_ERROR;
 			}
@@ -589,17 +847,8 @@ retval_t network_mysqld_write_len(network_mysqld *UNUSED_PARAM(srv), network_soc
 
 retval_t network_mysqld_write(network_mysqld *srv, network_socket *con) {
 	retval_t ret;
-	int corked;
 
-#ifdef TCP_CORK
-	corked = 1;
-	setsockopt(con->fd, IPPROTO_TCP, TCP_CORK, &corked, sizeof(corked));
-#endif
 	ret = network_mysqld_write_len(srv, con, -1);
-#ifdef TCP_CORK
-	corked = 0;
-	setsockopt(con->fd, IPPROTO_TCP, TCP_CORK, &corked, sizeof(corked));
-#endif
 
 	return ret;
 }
@@ -608,6 +857,11 @@ retval_t network_mysqld_write(network_mysqld *srv, network_socket *con) {
  * call the hooks of the plugins for each state
  *
  * if the plugin doesn't implement a hook, we provide a default operation
+ *
+ * @param srv      the global context
+ * @param con      the connection context
+ * @param state    state to handle
+ * @return         RET_SUCCESS on success
  */
 retval_t plugin_call(network_mysqld *srv, network_mysqld_con *con, int state) {
 	NETWORK_MYSQLD_PLUGIN_FUNC(func) = NULL;
@@ -628,7 +882,6 @@ retval_t plugin_call(network_mysqld *srv, network_mysqld_con *con, int state) {
 		}
 
 		break;
-
 	case CON_STATE_SEND_HANDSHAKE:
 		func = con->plugins.con_send_handshake;
 
@@ -748,6 +1001,10 @@ retval_t plugin_call(network_mysqld *srv, network_mysqld_con *con, int state) {
 
 /**
  * handle the different states of the MySQL protocol
+ *
+ * @param event_fd     fd on which the event was fired
+ * @param events       the event that was fired
+ * @param user_data    the connection handle
  */
 void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 	guint ostate;
@@ -788,6 +1045,9 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 	event_base_set(srv->event_base, &(ev_struct->event));\
 	event_add(&(ev_struct->event), timeout);
 
+	/**
+	 * loop on the same connection as long as we don't end up in a stable state
+	 */
 	do {
 		ostate = con->state;
 		switch (con->state) {
@@ -830,12 +1090,34 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 		case CON_STATE_CONNECT_SERVER:
 			switch (plugin_call(srv, con, con->state)) {
 			case RET_SUCCESS:
-				g_assert(con->server);
+
+				/**
+				 * hmm, if this is success and we have something in the clients send-queue
+				 * we just send it out ... who needs a server ? */
+
+				if (con->client->send_queue->chunks->length > 0 && con->server == NULL) {
+					/* we want to send something to the client */
+
+					con->state = CON_STATE_SEND_HANDSHAKE;
+				} else {
+					g_assert(con->server);
+				}
 
 				break;
 			case RET_ERROR_RETRY:
-				/* hack to force a retry */
-				ostate = CON_STATE_INIT;
+				if (con->server) {
+					/**
+					 * we have a server connection waiting to begin writable
+					 */
+					WAIT_FOR_EVENT(con->server, EV_WRITE, NULL);
+					return;
+				} else {
+					/* try to get a connection to another backend,
+					 *
+					 * setting ostate = CON_STATE_INIT is a hack to make sure
+					 * the loop is coming back to this function again */
+					ostate = CON_STATE_INIT;
+				}
 
 				break;
 			case RET_ERROR:
@@ -908,7 +1190,10 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				return;
 			case RET_ERROR_RETRY:
 			case RET_ERROR:
-				g_error("%s.%d: network_mysqld_write(CON_STATE_SEND_HANDSHAKE) returned an error", __FILE__, __LINE__);
+				/**
+				 * writing failed, closing connection
+				 */
+				con->state = CON_STATE_ERROR;
 				break;
 			}
 
@@ -963,8 +1248,9 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 			case RET_ERROR_RETRY:
 			case RET_ERROR:
 				/* might be a connection close, we should just close the connection and be happy */
-				g_error("%s.%d: network_mysqld_write(CON_STATE_SEND_AUTH) returned an error", __FILE__, __LINE__);
-				return;
+				con->state = CON_STATE_ERROR;
+
+				break;
 			}
 
 			switch (plugin_call(srv, con, con->state)) {
@@ -1032,8 +1318,10 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				return;
 			case RET_ERROR_RETRY:
 			case RET_ERROR:
-				g_error("%s.%d: network_mysqld_write(CON_STATE_SEND_AUTH_RESULT) returned an error", __FILE__, __LINE__);
-				return;
+				g_debug("%s.%d: network_mysqld_write(CON_STATE_SEND_AUTH_RESULT) returned an error", __FILE__, __LINE__);
+
+				con->state = CON_STATE_ERROR;
+				break;
 			}
 
 			switch (plugin_call(srv, con, con->state)) {
@@ -1082,8 +1370,9 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 			case RET_ERROR_RETRY:
 			case RET_ERROR:
 				/* might be a connection close, we should just close the connection and be happy */
-				g_error("%s.%d: network_mysqld_write(CON_STATE_SEND_AUTH_OLD_PASSWORD) returned an error", __FILE__, __LINE__);
-				return;
+				g_debug("%s.%d: network_mysqld_write(CON_STATE_SEND_AUTH_OLD_PASSWORD) returned an error", __FILE__, __LINE__);
+				con->state = CON_STATE_ERROR;
+				break;
 			}
 
 			switch (plugin_call(srv, con, con->state)) {
@@ -1197,8 +1486,13 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				return;
 			case RET_ERROR_RETRY:
 			case RET_ERROR:
-				g_error("%s.%d: network_mysqld_write(CON_STATE_SEND_QUERY) returned an error", __FILE__, __LINE__);
-				return;
+				g_debug("%s.%d: network_mysqld_write(CON_STATE_SEND_QUERY) returned an error", __FILE__, __LINE__);
+
+				/**
+				 * write() failed, close the connections 
+				 */
+				con->state = CON_STATE_ERROR;
+				break;
 			}
 
 			if (con->is_overlong_packet) {
@@ -1275,8 +1569,11 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 				return;
 			case RET_ERROR_RETRY:
 			case RET_ERROR:
-				g_critical("%s.%d: network_mysqld_write(CON_STATE_SEND_QUERY_RESULT) returned an error", __FILE__, __LINE__);
-
+				/**
+				 * client is gone away
+				 *
+				 * close the connection and clean up
+				 */
 				con->state = CON_STATE_ERROR;
 				break;
 			}
@@ -1326,9 +1623,14 @@ void network_mysqld_con_handle(int event_fd, short events, void *user_data) {
 }
 
 /**
- * we will be called by the event handler 
+ * accept a connection
  *
+ * event handler for listening connections
  *
+ * @param event_fd     fd on which the event was fired
+ * @param events       the event that was fired
+ * @param user_data    the listening connection handle
+ * 
  */
 void network_mysqld_con_accept(int event_fd, short events, void *user_data) {
 	network_mysqld_con *con = user_data;
@@ -1336,7 +1638,6 @@ void network_mysqld_con_accept(int event_fd, short events, void *user_data) {
 	socklen_t addr_len;
 	struct sockaddr_in ipv4;
 	int fd;
-	int ioctlvar;
 
 	g_assert(events == EV_READ);
 	g_assert(con->server);
@@ -1346,12 +1647,8 @@ void network_mysqld_con_accept(int event_fd, short events, void *user_data) {
 	if (-1 == (fd = accept(event_fd, (struct sockaddr *)&ipv4, &addr_len))) {
 		return ;
 	}
-#ifdef _WIN32
-	ioctlvar = 1;
-	ioctlsocket(fd, FIONBIO, &ioctlvar);
-#else
-	fcntl(fd, F_SETFL, O_NONBLOCK | O_RDWR);
-#endif
+
+	network_mysqld_con_set_non_blocking(fd);
 
 	/* looks like we open a client connection */
 	client_con = network_mysqld_con_init(con->srv);
@@ -1377,7 +1674,11 @@ void network_mysqld_con_accept(int event_fd, short events, void *user_data) {
 	}
 
 
-	/* copy the config */
+	/* copy the config
+	 *
+	 * @todo replace network-type by a function pointer for the init 
+	 *
+	 */
 	client_con->config = con->config;
 	client_con->config.network_type = con->config.network_type;
 
@@ -1399,7 +1700,10 @@ void network_mysqld_con_accept(int event_fd, short events, void *user_data) {
 }
 
 
-void handle_timeout() {
+/**
+ * timeout handler for the event-loop 
+ */
+static void handle_timeout() {
 	if (!agent_shutdown) return;
 
 	/* we have to shutdown, disable all events to leave the dispatch */
@@ -1471,6 +1775,9 @@ void *network_mysqld_thread(void *_srv) {
 		proxy_con = con;
 	}
 
+	/**
+	 * check once a second if we shall shutdown the proxy
+	 */
 	while (!agent_shutdown) {
 		struct timeval timeout;
 		int r;
@@ -1508,6 +1815,9 @@ void *network_mysqld_thread(void *_srv) {
 	return NULL;
 }
 
+/**
+ * @todo move to network_mysqld_proto
+ */
 int network_mysqld_con_send_resultset(network_socket *con, GPtrArray *fields, GPtrArray *rows) {
 	GString *s;
 	gsize i, j;
